use regex_macro::regex;

default_derive!{
    pub struct AttributeId {
        id: String
    }
}

impl TryFrom<&str> for AttributeId {
    type Error = &'static str;

    fn try_from(value: &str) -> Result<Self, Self::Error> {
        let re = regex!("^[[:digit:]]+$");
        if re.is_match(value) && value.len() <= 10 {
            Ok(AttributeId{ id: value.to_string() })
        }
        else {
            Err("Failed to parse AttributeId")
        }
    }
}

impl Into<String> for AttributeId {
    fn into(self) -> String {
        self.id
    }
}

#[test]
fn empty() {
    let id: Result<AttributeId, _> = "".try_into();
    assert!(id.is_err())
}

#[test]
fn oversized() {
    let id: Result<AttributeId, _> = "12345678910".try_into();
    assert!(id.is_err())
}

#[test]
fn forbidden_char() {
    let id: Result<AttributeId, _> = "123r5".try_into();
    assert!(id.is_err())
}

#[test]
fn valid1() {
    let id: Result<AttributeId, _> = "0".try_into();
    assert_eq!(id, Ok(AttributeId{id: String::from("0")}))
}

#[test]
fn valid2() {
    let id: Result<AttributeId, _> = "123456789".try_into();
    assert_eq!(id, Ok(AttributeId{id: String::from("123456789")}));
}